import pandas as pd
import numpy as np
import datetime
import time
from matplotlib import pyplot as plt
from LIB.MIDDLE.CellStateEstimation.Common.V1_0_1 import BatParam

class SafetyWarning:
    def __init__(self,sn,celltype,df_short,df_liplated,df_uniform,OutLineVol_Rate):  #参数初始化

        self.sn=sn
        self.celltype=celltype
        self.param=BatParam.BatParam(celltype)
        self.df_short=df_short
        self.df_liplated=df_liplated
        self.df_uniform=df_uniform.dropna(axis=0,how='any')
        self.OutLineVol_Rate=OutLineVol_Rate
    
    def diag(self):
        if self.celltype<=50:
            df_res=self._warning_diag()
            return df_res    
        else:
            df_res=self._warning_diag()
            return df_res
        

    #电池热安全预警诊断功能.................................................................................................
    def _warning_diag(self):

        df_res=pd.DataFrame(columns=['start_time', 'end_time', 'product_id', 'code', 'level', 'info','advice'])

        time_now=datetime.datetime.now()
        time_sp='0000-00-00 00:00:00'

        #参数初始化.......................................
        shortfault=0
        liplatedfault=0
        uniformfault=0
        volt_rate=[]

        if not self.df_short.empty:
            short_current=self.df_short['short_current']
            short_current=short_current.str.replace("[", '')
            short_current=short_current.str.replace("]", '')
        
        if not self.OutLineVol_Rate.empty:
            volt_column = ['单体电压'+str(i) for i in range(1,self.param.CellVoltNums+1)]
            self.OutLineVol_Rate['VolChng_Uni']=self.OutLineVol_Rate['VolChng_Uni'].str.replace("[","")
            self.OutLineVol_Rate['VolChng_Uni']=self.OutLineVol_Rate['VolChng_Uni'].str.replace("]","")
            self.OutLineVol_Rate['VolOl_Uni']=self.OutLineVol_Rate['VolOl_Uni'].str.replace("[","")
            self.OutLineVol_Rate['VolOl_Uni']=self.OutLineVol_Rate['VolOl_Uni'].str.replace("]","")
            VoltChangeOutLine_Rate=self.OutLineVol_Rate['VolChng_Uni'].str.split(',',expand=True)
            Volt_3Sigma=self.OutLineVol_Rate['VolOl_Uni'].str.split(',',expand=True)
            VoltChangeOutLine_Rate.columns=volt_column
            Volt_3Sigma.columns=volt_column
            VoltChangeOutLine_Rate['time']=self.OutLineVol_Rate['time']
            time0=time.mktime(VoltChangeOutLine_Rate.loc[0,'time'].timetuple())
            for i in range(0,len(VoltChangeOutLine_Rate)):
                VoltChangeOutLine_Rate.loc[i,'time']=(time.mktime(VoltChangeOutLine_Rate.loc[i,'time'].timetuple())-time0)/60000
        
        if not self.df_liplated.empty:
            liplated=self.df_liplated['liplated_amount']
            liplated=liplated.str.replace("[", '')
            liplated=liplated.str.replace("]", '')
        
        if not self.df_uniform.empty:
            cellvolt_rank=self.df_uniform['cellvolt_rank']
            cellvolt_rank=cellvolt_rank.str.replace("[", '')
            cellvolt_rank=cellvolt_rank.str.replace("]", '')

        for i in range(self.param.CellVoltNums):

            #漏电流故障判断...........................................................................
            if not self.df_short.empty:
                self.df_short['cellshort'+str(i+1)]=short_current.map(lambda x:eval(x.split(',')[i]))

                cellshort=self.df_short['cellshort'+str(i+1)]
                index_list=cellshort[cellshort<self.param.LeakCurrentLv2].index
                if len(index_list)==2 and (index_list[1]-index_list[0])==1:
                    shortfault=1
                elif len(index_list)>3:
                    shortfault=1
                else:
                    shortfault=0
            
            #电压变化率及电压离群度.................................................................................
            if not self.OutLineVol_Rate.empty and VoltChangeOutLine_Rate.iloc[-1]['time']*60000>24*3600 and len(VoltChangeOutLine_Rate)>10:
                VoltChangeOutLine_Rate[volt_column[i]]=VoltChangeOutLine_Rate[volt_column[i]].map(lambda x:eval(x))
                volt3sigma=np.array(Volt_3Sigma[volt_column[i]].map(lambda x:eval(x)))
                volt3sigma_sum=np.sum(volt3sigma<-3)
                a1,b1=np.polyfit(VoltChangeOutLine_Rate['time'].tolist(),VoltChangeOutLine_Rate[volt_column[i]].tolist(),1)
                y=a1*VoltChangeOutLine_Rate['time']+b1
                volt_rate.append(a1)
                # plt.plot(VoltChangeOutLine_Rate['time'],y,label=volt_column[i])
                # plt.scatter( VoltChangeOutLine_Rate['time'],VoltChangeOutLine_Rate[volt_column[i]],marker='o')
                # plt.legend(loc='best')
                # plt.rcParams['font.sans-serif']=['SimHei'] #用来正常显示中文标签
                # plt.rcParams['axes.unicode_minus']=False #用来正常显示负号  
                # plt.show()
                
                if a1<self.param.TrwVoltRate:
                    voltratefault=1
                else:
                    voltratefault=0
                if volt3sigma_sum>len(self.OutLineVol_Rate)/2:
                    voltsigmafault=1
                else:
                    voltsigmafault=0
            else:
                voltratefault=0
                voltsigmafault=0
            
            #析锂故障判断...............................................................................
            # if not self.df_liplated.empty:
            #     self.df_liplated['liplated_amount'+str(i+1)]=liplated.map(lambda x:eval(x.split(',')[i]))

            #     if max(self.df_liplated['liplated_amount'+str(i+1)])>30:
            #         liplatedfault=1
            #     else:
            #         liplatedfault=0
            # else:
            #     liplatedfault=0

            #电芯SOC排名判断.............................................................................
            if not self.df_uniform.empty:
                self.df_uniform['cellvolt_rank'+str(i+1)]=cellvolt_rank.map(lambda x:eval(x.split(',')[i]))
                
                if max(self.df_uniform['cellvolt_rank'+str(i+1)])<5:
                    uniformfault=1
                else:
                    uniformfault=0
            else:
                uniformfault=0
            
            #电池热安全预警...............................................................................
            if (shortfault==1 or voltratefault==1)and (voltsigmafault==1 or uniformfault==1):
                faultcode=110
                faultlv=4
                faultinfo='电芯{}发生热失控安全预警'.format(i+1)
                faultadvice='1联系用户远离电池，立刻召回电池'
                df_res.loc[0]=[time_now, time_sp, self.sn, faultcode, faultlv, faultinfo, faultadvice]
            elif voltratefault==1:
                faultcode=110
                faultlv=4
                faultinfo='电芯{}发生热失控安全预警'.format(i+1)
                faultadvice='2联系用户远离电池，立刻召回电池'
                df_res.loc[0]=[time_now, time_sp, self.sn, faultcode, faultlv, faultinfo, faultadvice]
            elif shortfault==1:
                faultcode=110
                faultlv=4
                faultinfo='电芯{}发生热失控安全预警'.format(i+1)
                faultadvice='3联系用户远离电池，立刻召回电池'
                df_res.loc[0]=[time_now, time_sp, self.sn, faultcode, faultlv, faultinfo, faultadvice]
            else:
                pass
        
        # print(volt_rate)
            
        return df_res
